@@ -1,4 +1,5 @@
 import autofit as af
+from autoarray.inversion import pixelizations as pix
 from autoarray.exc import PixelizationException, InversionException, GridException
 from autofit.exc import FitException
 from autogalaxy.galaxy import galaxy as g
@@ -10,9 +11,7 @@ from autolens.pipeline.phase.dataset import analysis as analysis_dataset
 
 
 class Analysis(ag_analysis.Analysis, analysis_dataset.Analysis):
-    def __init__(
-        self, masked_interferometer, settings, cosmology, image_path=None, results=None
-    ):
+    def __init__(self, masked_interferometer, settings, cosmology, results=None):
 
         super(Analysis, self).__init__(
             masked_dataset=masked_interferometer,
@@ -22,12 +21,7 @@ class Analysis(ag_analysis.Analysis, analysis_dataset.Analysis):
         )
 
         self.visualizer = visualizer.PhaseInterferometerVisualizer(
-            masked_dataset=masked_interferometer, image_path=image_path
-        )
-
-        self.visualizer.visualize_hyper_images(
-            hyper_galaxy_image_path_dict=self.hyper_galaxy_image_path_dict,
-            hyper_model_image=self.hyper_model_image,
+            masked_dataset=masked_interferometer
         )
 
         result = ag_analysis.last_result_with_use_as_hyper_dataset(results=results)
@@ -136,7 +130,61 @@ class Analysis(ag_analysis.Analysis, analysis_dataset.Analysis):
             settings_inversion=self.settings.settings_inversion,
         )
 
-    def visualize(self, instance, during_analysis):
+    def stochastic_log_evidences_for_instance(self, instance):
+
+        instance = self.associate_hyper_images(instance=instance)
+        tracer = self.tracer_for_instance(instance=instance)
+
+        if not tracer.has_pixelization:
+            return None
+
+        if not isinstance(
+            tracer.pixelizations_of_planes[-1], pix.VoronoiBrightnessImage
+        ):
+            return None
+
+        hyper_background_noise = self.hyper_background_noise_for_instance(
+            instance=instance
+        )
+
+        settings_pixelization = (
+            self.settings.settings_pixelization.settings_with_is_stochastic_true()
+        )
+
+        log_evidences = []
+
+        for i in range(self.settings.settings_lens.stochastic_samples):
+
+            try:
+                log_evidence = fit.FitInterferometer(
+                    masked_interferometer=self.masked_dataset,
+                    tracer=tracer,
+                    hyper_background_noise=hyper_background_noise,
+                    settings_pixelization=settings_pixelization,
+                    settings_inversion=self.settings.settings_inversion,
+                ).log_evidence
+            except (
+                PixelizationException,
+                InversionException,
+                GridException,
+                OverflowError,
+            ) as e:
+                log_evidence = None
+
+            if log_evidence is not None:
+                log_evidences.append(log_evidence)
+
+        return log_evidences
+
+    def visualize(self, paths: af.Paths, instance, during_analysis):
+
+        self.visualizer.visualize_interferometer(paths=paths)
+
+        self.visualizer.visualize_hyper_images(
+            paths=paths,
+            hyper_galaxy_image_path_dict=self.hyper_galaxy_image_path_dict,
+            hyper_model_image=self.hyper_model_image,
+        )
 
         self.associate_hyper_images(instance=instance)
         tracer = self.tracer_for_instance(instance=instance)
@@ -166,9 +214,9 @@ class Analysis(ag_analysis.Analysis, analysis_dataset.Analysis):
             visualizer = self.visualizer
 
         visualizer.visualize_ray_tracing(
-            tracer=fit.tracer, during_analysis=during_analysis
+            paths=paths, tracer=fit.tracer, during_analysis=during_analysis
         )
-        visualizer.visualize_fit(fit=fit, during_analysis=during_analysis)
+        visualizer.visualize_fit(paths=paths, fit=fit, during_analysis=during_analysis)
 
     def make_attributes(self):
         return Attributes(
