@@ -1,8 +1,10 @@
 import copy
 from scipy.stats import norm
 import matplotlib.pyplot as plt
+from os import path
 
 from autoconf import conf
+import autofit as af
 import autoarray as aa
 from autoarray.plot import mat_objs
 from autogalaxy.plot import fit_galaxy_plots, hyper_plots, inversion_plots
@@ -19,14 +21,8 @@ def plot_setting(section, name):
 
 
 class AbstractVisualizer:
-    def __init__(self, image_path):
+    def __init__(self):
 
-        self.plotter = lensing_plotters.Plotter(
-            output=mat_objs.Output(path=image_path, format="png")
-        )
-        self.sub_plotter = lensing_plotters.SubPlotter(
-            output=mat_objs.Output(path=image_path + "subplots/", format="png")
-        )
         self.include = lensing_plotters.Include()
 
         self.plot_ray_tracing_all_at_end_png = plot_setting(
@@ -49,6 +45,27 @@ class AbstractVisualizer:
             "ray_tracing", "magnification"
         )
 
+    @staticmethod
+    def plotter_from_paths(paths: af.Paths, subfolders=None, format="png"):
+        if subfolders is None:
+            return lensing_plotters.Plotter(
+                output=mat_objs.Output(path=paths.image_path, format=format)
+            )
+        return lensing_plotters.Plotter(
+            output=mat_objs.Output(
+                path=path.join(paths.image_path, subfolders), format=format
+            )
+        )
+
+    @staticmethod
+    def sub_plotter_from_paths(paths: af.Paths):
+
+        return lensing_plotters.SubPlotter(
+            output=mat_objs.Output(
+                path=path.join(paths.image_path, "subplots"), format="png"
+            )
+        )
+
     def new_visualizer_with_preloaded_critical_curves_and_caustics(
         self, preloaded_critical_curves, preloaded_caustics
     ):
@@ -64,8 +81,10 @@ class AbstractVisualizer:
 
 
 class PhaseGalaxyVisualizer(AbstractVisualizer):
-    def __init__(self, image_path):
-        super().__init__(image_path)
+    def __init__(self):
+
+        super().__init__()
+
         self.plot_galaxy_fit_all_at_end_png = plot_setting(
             "galaxy_fit", "all_at_end_png"
         )
@@ -81,13 +100,18 @@ class PhaseGalaxyVisualizer(AbstractVisualizer):
             "galaxy_fit", "chi_squared_map"
         )
 
-    def plot_galaxy_fit_subplot(self, fit):
+    def plot_galaxy_fit_subplot(self, paths: af.Paths, fit):
+
+        sub_plotter = self.sub_plotter_from_paths(paths=paths)
+
         if self.plot_subplot_galaxy_fit:
             fit_galaxy_plots.subplot_fit_galaxy(
-                fit=fit, include=self.include, sub_plotter=self.sub_plotter
+                fit=fit, include=self.include, sub_plotter=sub_plotter
             )
 
-    def plot_fit_individuals(self, fit):
+    def plot_fit_individuals(self, paths: af.Paths, fit):
+
+        plotter = self.plotter_from_paths(paths=paths)
 
         fit_galaxy_plots.individuals(
             fit=fit,
@@ -97,13 +121,15 @@ class PhaseGalaxyVisualizer(AbstractVisualizer):
             plot_residual_map=self.plot_galaxy_fit_residual_map,
             plot_chi_squared_map=self.plot_galaxy_fit_chi_squared_map,
             include=self.include,
-            plotter=self.plotter,
+            plotter=plotter,
         )
 
 
 class PhaseDatasetVisualizer(AbstractVisualizer):
-    def __init__(self, masked_dataset, image_path):
-        super().__init__(image_path)
+    def __init__(self, masked_dataset):
+
+        super().__init__()
+
         self.masked_dataset = masked_dataset
 
         self.plot_subplot_dataset = plot_setting("dataset", "subplot_dataset")
@@ -180,11 +206,10 @@ class PhaseDatasetVisualizer(AbstractVisualizer):
 
         self.plot_stochastic_histogram = plot_setting("other", "stochastic_histogram")
 
-    def visualize_ray_tracing(self, tracer, during_analysis):
+    def visualize_ray_tracing(self, paths: af.Paths, tracer, during_analysis):
 
-        plotter = self.plotter.plotter_with_new_output(
-            path=self.plotter.output.path + "ray_tracing/"
-        )
+        plotter = self.plotter_from_paths(paths=paths, subfolders="ray_tracing")
+        sub_plotter = self.sub_plotter_from_paths(paths=paths)
 
         if self.plot_subplot_ray_tracing:
 
@@ -195,7 +220,7 @@ class PhaseDatasetVisualizer(AbstractVisualizer):
                     masked_dataset=self.masked_dataset
                 ),
                 include=self.include,
-                sub_plotter=self.sub_plotter,
+                sub_plotter=sub_plotter,
             )
 
         ray_tracing_plots.individual(
@@ -235,12 +260,12 @@ class PhaseDatasetVisualizer(AbstractVisualizer):
 
             if self.plot_ray_tracing_all_at_end_fits:
 
-                self.visualize_ray_tracing_in_fits(tracer=tracer)
+                self.visualize_ray_tracing_in_fits(paths=paths, tracer=tracer)
 
-    def visualize_ray_tracing_in_fits(self, tracer):
+    def visualize_ray_tracing_in_fits(self, paths: af.Paths, tracer):
 
-        fits_plotter = self.plotter.plotter_with_new_output(
-            path=self.plotter.output.path + "/ray_tracing/fits/", format="fits"
+        fits_plotter = self.plotter_from_paths(
+            paths=paths, subfolders=path.join("ray_tracing", "fits"), format="fits"
         )
 
         ray_tracing_plots.individual(
@@ -255,15 +280,12 @@ class PhaseDatasetVisualizer(AbstractVisualizer):
             plotter=fits_plotter,
         )
 
-    def visualize_hyper_images(self, hyper_galaxy_image_path_dict, hyper_model_image):
+    def visualize_hyper_images(
+        self, paths: af.Paths, hyper_galaxy_image_path_dict, hyper_model_image
+    ):
 
-        plotter = self.plotter.plotter_with_new_output(
-            path=self.plotter.output.path + "hyper/"
-        )
-
-        sub_plotter = self.sub_plotter.plotter_with_new_output(
-            path=self.plotter.output.path + "hyper/"
-        )
+        plotter = self.plotter_from_paths(paths=paths, subfolders="hyper")
+        sub_plotter = self.sub_plotter_from_paths(paths=paths)
 
         if self.plot_hyper_model_image:
             hyper_plots.hyper_model_image(
@@ -286,17 +308,15 @@ class PhaseDatasetVisualizer(AbstractVisualizer):
             )
 
     def visualize_stochastic_histogram(
-        self, log_evidences, max_log_evidence, during_analysis, histogram_bins=10
+        self, paths: af.Paths, log_evidences, max_log_evidence, histogram_bins=10
     ):
 
         if log_evidences is None:
             return
 
-        plotter = self.plotter.plotter_with_new_output(
-            path=self.plotter.output.path + "other/"
-        )
+        plotter = self.plotter_from_paths(paths=paths, subfolders="other")
 
-        if self.plot_stochastic_histogram and not during_analysis:
+        if self.plot_stochastic_histogram:
 
             (mu, sigma) = norm.fit(log_evidences)
             n, bins, patches = plt.hist(x=log_evidences, bins=histogram_bins, density=1)
@@ -306,32 +326,29 @@ class PhaseDatasetVisualizer(AbstractVisualizer):
             plt.title("Stochastic Log Evidence Histogram")
             plt.axvline(max_log_evidence, color="r")
             plt.savefig(
-                f"{plotter.output.path}stochastic_histogram.png", bbox_inches="tight"
+                path.join(plotter.output.path, "stochastic_histogram.png"),
+                bbox_inches="tight",
             )
+            plt.close()
 
 
 class PhaseImagingVisualizer(PhaseDatasetVisualizer):
-    def __init__(self, masked_dataset, image_path, results=None):
-        super(PhaseImagingVisualizer, self).__init__(
-            masked_dataset=masked_dataset, image_path=image_path
-        )
+    def __init__(self, masked_dataset):
+        super(PhaseImagingVisualizer, self).__init__(masked_dataset=masked_dataset)
 
         self.plot_dataset_psf = plot_setting("dataset", "psf")
 
         self.plot_hyper_model_image = plot_setting("hyper_galaxy", "model_image")
         self.plot_hyper_galaxy_images = plot_setting("hyper_galaxy", "images")
 
-        self.visualize_imaging()
-
     @property
     def masked_imaging(self):
         return self.masked_dataset
 
-    def visualize_imaging(self):
+    def visualize_imaging(self, paths: af.Paths):
 
-        plotter = self.plotter.plotter_with_new_output(
-            path=self.plotter.output.path + "imaging/"
-        )
+        plotter = self.plotter_from_paths(paths=paths, subfolders="imaging")
+        sub_plotter = self.sub_plotter_from_paths(paths=paths)
 
         if self.plot_subplot_dataset:
             aa.plot.Imaging.subplot_imaging(
@@ -343,7 +360,7 @@ class PhaseImagingVisualizer(PhaseDatasetVisualizer):
                     masked_dataset=self.masked_dataset
                 ),
                 include=self.include,
-                sub_plotter=self.sub_plotter,
+                sub_plotter=sub_plotter,
             )
 
         aa.plot.Imaging.individual(
@@ -365,20 +382,19 @@ class PhaseImagingVisualizer(PhaseDatasetVisualizer):
             plotter=plotter,
         )
 
-    def visualize_fit(self, fit, during_analysis):
+    def visualize_fit(self, paths: af.Paths, fit, during_analysis):
 
-        plotter = self.plotter.plotter_with_new_output(
-            path=self.plotter.output.path + "fit_imaging/"
-        )
+        plotter = self.plotter_from_paths(paths=paths, subfolders="fit_imaging")
+        sub_plotter = self.sub_plotter_from_paths(paths=paths)
 
         if self.plot_subplot_fit:
             fit_imaging_plots.subplot_fit_imaging(
-                fit=fit, include=self.include, sub_plotter=self.sub_plotter
+                fit=fit, include=self.include, sub_plotter=sub_plotter
             )
 
         if self.plot_subplots_of_all_planes_fits:
             fit_imaging_plots.subplots_of_all_planes(
-                fit=fit, include=self.include, sub_plotter=self.sub_plotter
+                fit=fit, include=self.include, sub_plotter=sub_plotter
             )
 
         fit_imaging_plots.individuals(
@@ -420,12 +436,10 @@ class PhaseImagingVisualizer(PhaseDatasetVisualizer):
                     ),
                     caustics=self.include.caustics_from_obj(obj=fit.tracer),
                     include=self.include,
-                    sub_plotter=self.sub_plotter,
+                    sub_plotter=sub_plotter,
                 )
 
-            plotter = self.plotter.plotter_with_new_output(
-                path=self.plotter.output.path + "inversion/"
-            )
+            plotter = self.plotter_from_paths(paths=paths, subfolders="inversion")
 
             inversion_plots.individuals(
                 inversion=fit.inversion,
@@ -458,6 +472,9 @@ class PhaseImagingVisualizer(PhaseDatasetVisualizer):
         if not during_analysis:
 
             if self.plot_fit_all_at_end_png:
+
+                plotter = self.plotter_from_paths(paths=paths, subfolders="fit_imaging")
+
                 fit_imaging_plots.individuals(
                     fit=fit,
                     plot_image=True,
@@ -475,6 +492,11 @@ class PhaseImagingVisualizer(PhaseDatasetVisualizer):
                 )
 
                 if fit.inversion is not None:
+
+                    plotter = self.plotter_from_paths(
+                        paths=paths, subfolders="inversion"
+                    )
+
                     inversion_plots.individuals(
                         inversion=fit.inversion,
                         image_positions=self.include.positions_from_fit(fit=fit),
@@ -509,12 +531,12 @@ class PhaseImagingVisualizer(PhaseDatasetVisualizer):
 
             if self.plot_fit_all_at_end_fits:
 
-                self.visualize_fit_in_fits(fit=fit)
+                self.visualize_fit_in_fits(paths=paths, fit=fit)
 
-    def visualize_fit_in_fits(self, fit):
+    def visualize_fit_in_fits(self, paths: af.Paths, fit):
 
-        fits_plotter = self.plotter.plotter_with_new_output(
-            path=self.plotter.output.path + "fit_imaging/fits/", format="fits"
+        fits_plotter = self.plotter_from_paths(
+            paths=paths, subfolders=path.join("fit_imaging", "fits"), format="fits"
         )
 
         fit_imaging_plots.individuals(
@@ -535,8 +557,8 @@ class PhaseImagingVisualizer(PhaseDatasetVisualizer):
 
         if fit.inversion is not None:
 
-            fits_plotter = self.plotter.plotter_with_new_output(
-                path=self.plotter.output.path + "inversion/fits/", format="fits"
+            fits_plotter = self.plotter_from_paths(
+                paths=paths, subfolders=path.join("inversion", "fits"), format="fits"
             )
 
             inversion_plots.individuals(
@@ -550,9 +572,9 @@ class PhaseImagingVisualizer(PhaseDatasetVisualizer):
 
 
 class PhaseInterferometerVisualizer(PhaseDatasetVisualizer):
-    def __init__(self, masked_dataset, image_path):
+    def __init__(self, masked_dataset):
         super(PhaseInterferometerVisualizer, self).__init__(
-            masked_dataset=masked_dataset, image_path=image_path
+            masked_dataset=masked_dataset
         )
 
         self.plot_dataset_uv_wavelengths = plot_setting("dataset", "uv_wavelengths")
@@ -560,25 +582,23 @@ class PhaseInterferometerVisualizer(PhaseDatasetVisualizer):
         self.plot_hyper_model_image = plot_setting("hyper_galaxy", "model_image")
         self.plot_hyper_galaxy_images = plot_setting("hyper_galaxy", "images")
 
-        self.visualize_interferometer()
-
     @property
     def masked_interferometer(self):
         return self.masked_dataset
 
-    def visualize_interferometer(self):
+    def visualize_interferometer(self, paths: af.Paths):
 
-        plotter = self.plotter.plotter_with_new_output(
-            path=self.plotter.output.path + "interferometer/"
-        )
+        sub_plotter = self.sub_plotter_from_paths(paths=paths)
 
         if self.plot_subplot_dataset:
             aa.plot.Interferometer.subplot_interferometer(
                 interferometer=self.masked_dataset.interferometer,
                 include=self.include,
-                sub_plotter=self.sub_plotter,
+                sub_plotter=sub_plotter,
             )
 
+        plotter = self.plotter_from_paths(paths=paths, subfolders="interferometer")
+
         aa.plot.Interferometer.individual(
             interferometer=self.masked_dataset.interferometer,
             plot_visibilities=self.plot_dataset_data,
@@ -588,21 +608,22 @@ class PhaseInterferometerVisualizer(PhaseDatasetVisualizer):
             plotter=plotter,
         )
 
-    def visualize_fit(self, fit, during_analysis):
+    def visualize_fit(self, paths: af.Paths, fit, during_analysis):
 
-        plotter = self.plotter.plotter_with_new_output(
-            path=self.plotter.output.path + "fit_interferometer/"
-        )
+        sub_plotter = self.sub_plotter_from_paths(paths=paths)
 
         if self.plot_subplot_fit:
+
             fit_interferometer_plots.subplot_fit_interferometer(
-                fit=fit, include=self.include, sub_plotter=self.sub_plotter
+                fit=fit, include=self.include, sub_plotter=sub_plotter
             )
 
             fit_interferometer_plots.subplot_fit_real_space(
-                fit=fit, include=self.include, sub_plotter=self.sub_plotter
+                fit=fit, include=self.include, sub_plotter=sub_plotter
             )
 
+        plotter = self.plotter_from_paths(paths=paths, subfolders="fit_interferometer")
+
         fit_interferometer_plots.individuals(
             fit=fit,
             plot_visibilities=self.plot_fit_data,
@@ -618,9 +639,7 @@ class PhaseInterferometerVisualizer(PhaseDatasetVisualizer):
 
         if fit.inversion is not None:
 
-            plotter = self.plotter.plotter_with_new_output(
-                path=self.plotter.output.path + "inversion/"
-            )
+            plotter = self.plotter_from_paths(paths=paths, subfolders="inversion")
 
             # if self.plot_fit_inversion_as_subplot:
             #     inversion_plots.subplot_inversion(
@@ -673,6 +692,11 @@ class PhaseInterferometerVisualizer(PhaseDatasetVisualizer):
         if not during_analysis:
 
             if self.plot_fit_all_at_end_png:
+
+                plotter = self.plotter_from_paths(
+                    paths=paths, subfolders="fit_interferometer"
+                )
+
                 fit_interferometer_plots.individuals(
                     fit=fit,
                     plot_visibilities=True,
@@ -687,6 +711,11 @@ class PhaseInterferometerVisualizer(PhaseDatasetVisualizer):
                 )
 
                 if fit.inversion is not None:
+
+                    plotter = self.plotter_from_paths(
+                        paths=paths, subfolders="inversion"
+                    )
+
                     inversion_plots.individuals(
                         inversion=fit.inversion,
                         image_positions=self.include.positions_from_fit(fit=fit),
@@ -720,8 +749,11 @@ class PhaseInterferometerVisualizer(PhaseDatasetVisualizer):
                     )
 
             if self.plot_fit_all_at_end_fits:
-                fits_plotter = plotter.plotter_with_new_output(
-                    path=plotter.output.path + "/fits/", format="fits"
+
+                fits_plotter = self.plotter_from_paths(
+                    paths=paths,
+                    subfolders=path.join("fit_interferometer", "fits"),
+                    format="fits",
                 )
 
                 fit_interferometer_plots.individuals(
@@ -738,6 +770,12 @@ class PhaseInterferometerVisualizer(PhaseDatasetVisualizer):
                 )
 
                 if fit.inversion is not None:
+                    fits_plotter = self.plotter_from_paths(
+                        paths=paths,
+                        subfolders=path.join("inversion", "fits"),
+                        format="fits",
+                    )
+
                     inversion_plots.individuals(
                         inversion=fit.inversion,
                         image_positions=self.include.positions_from_fit(fit=fit),
@@ -766,18 +804,23 @@ class PhaseInterferometerVisualizer(PhaseDatasetVisualizer):
 
 
 class HyperGalaxyVisualizer(AbstractVisualizer):
-    def __init__(self, image_path):
-        super().__init__(image_path)
+    def __init__(self):
+        super().__init__()
         self.plot_hyper_galaxy_subplot = plot_setting(
             "hyper_galaxy", "subplot_hyper_galaxy"
         )
 
-    def visualize_hyper_galaxy(self, fit, hyper_fit, galaxy_image, contribution_map_in):
+    def visualize_hyper_galaxy(
+        self, paths: af.Paths, fit, hyper_fit, galaxy_image, contribution_map_in
+    ):
+
+        sub_plotter = self.sub_plotter_from_paths(paths=paths)
+
         hyper_plots.subplot_fit_hyper_galaxy(
             fit=fit,
             hyper_fit=hyper_fit,
             galaxy_image=galaxy_image,
             contribution_map_in=contribution_map_in,
             include=self.include,
-            sub_plotter=self.sub_plotter,
+            sub_plotter=sub_plotter,
         )
