@@ -12,7 +12,7 @@ directory = dirname(realpath(__file__))
 @pytest.fixture(name="config", autouse=True)
 def set_config_path():
     conf.instance = conf.Config(
-        path.join(directory, "config"), path.join(directory, "pipeline/output")
+        path.join(directory, "config"), path.join(directory, "pipeline", "output")
     )
     return conf.instance
 
