@@ -34,13 +34,14 @@ class TestCheckPositionsTrace:
 
         settings = al.SettingsLens(positions_threshold=50.0)
         settings.check_positions_trace_within_threshold_via_tracer(
-            tracer=tracer, positions=al.GridCoordinates([[(1.0, 1.0), (2.0, 2.0)]])
+            tracer=tracer, positions=al.GridIrregularGrouped([[(1.0, 1.0), (2.0, 2.0)]])
         )
 
         settings = al.SettingsLens(positions_threshold=0.0)
         with pytest.raises(exc.RayTracingException):
             settings.check_positions_trace_within_threshold_via_tracer(
-                tracer=tracer, positions=al.GridCoordinates([[(1.0, 1.0), (2.0, 2.0)]])
+                tracer=tracer,
+                positions=al.GridIrregularGrouped([[(1.0, 1.0), (2.0, 2.0)]]),
             )
 
         # No mass profile - doesnt raise exception
@@ -50,7 +51,7 @@ class TestCheckPositionsTrace:
         )
 
         settings.check_positions_trace_within_threshold_via_tracer(
-            tracer=tracer, positions=al.GridCoordinates([[(1.0, 1.0), (2.0, 2.0)]])
+            tracer=tracer, positions=al.GridIrregularGrouped([[(1.0, 1.0), (2.0, 2.0)]])
         )
 
         # Single plane - doesnt raise exception
@@ -60,5 +61,5 @@ class TestCheckPositionsTrace:
         )
 
         settings.check_positions_trace_within_threshold_via_tracer(
-            tracer=tracer, positions=al.GridCoordinates([[(1.0, 1.0), (2.0, 2.0)]])
+            tracer=tracer, positions=al.GridIrregularGrouped([[(1.0, 1.0), (2.0, 2.0)]])
         )
